#include "Autonomous.h"
#include <frc/smartdashboard/SmartDashboard.h>

const double kMotorSpeed = .3;
const double kLeftTurn = -90;
const double kRightTurn = 90;
const double kShootSpeed = .7;
const double kShooterSpeed = .75;
// Old Auto
/*const double kMoveToPowerPort = 101.75;                            // ONLY FOR STARTING RIGHT, NOT FOR LEFT AND CENTER. Move up to power port (Step 0)
const double kStepZeroDrive = 72;                                  // ONLY FOR STARTING CENTER AND LEFT, NOT FOR RIGHT. Moving up to the turn point (Step 0)
const double kStepFourDrive = (kMoveToPowerPort - kStepZeroDrive); // ONLY FOR STARTING CENTER AND LEFT, NOT FOR RIGHT. Moving up t o the wall (Step 0)
const double kStepTwoDriveLeft = 140;                              // Lining up with the power port if you start left (Step 2)
const double kStepTwoDriveCenter = 72;                             // Linging up with the power port if you start center (Step 2)
// const double kBackOffPowerPort = -160;
const double kBackOffPowerPort = -10.75; // (Step 6)
const double kTurnToTrench = 161.12;     // (Step 7)
const double kMoveToPanel = 194.4;
const double kTurnToPanel = (180 - kTurnToTrench);
const double kCollectPanelBalls = 108;
const double kLeaveControlPanel = (-(kCollectPanelBalls + 168));
const double kLineUpToPowerPort = 62.905; // Lines up to the Power Port after backing up from the control panel
const double kGoToPowerPort = (-26.68);*/

// Constants for starting at right wall (moveLeft())
const double kRightWallForwardZero = -6;
const double kRightWallTurnTwo = 51;
const double kRightWallForwardFour = -119.5;
const double kRightWallTurnSix = 39;
const double kRightWallForwardEight = -10;
// The following are for picking up trench balls after power port
const double kTrenchForwardZero = 10;
const double kTrenchTurnTwo = -39;
const double kTrenchForwardFour = 119.5;
const double kTrenchTurnSix = (-(kTrenchTurnTwo));
// The following are for picking up trench balls and shooting  (FBA - Five Ball Auto)
const double kFBAForwardTwo = 0; // Start moving to trench balls
const double kFBAForwardEight = 0; // Go backwards to line (shooting position)


Autonomous::Autonomous(Drive *drive, GamePiece *gamePiece)
{
    this->drive = drive;
    this->gamePiece = gamePiece;
}

void Autonomous::process()
{ // Checks what auto
    switch (currentAutoMode)
    {
    case DONOTHING:
        doNothing();
        break;
    case CROSS_PORT:
        crossLine();
        break;
    case DRIVE_RIGHT_MORE:
        driveRightMore();
        break;
    case SHOOT_FROM_LINE:
        shootLine();
        break;
    case JUST_LINE:
        justLine();
        break;
    case DRIVE_LEFT:
        driveLeft();
        break;
    case CROSS_PORT_TRENCH:
        crossLine();
        trenchBalls();
        break;
    case FIVE_BALL_AUTO:
        fiveBallAuto();
        break;
    case PATH_A:
        pathA();
        break;
    case BOUNCE_PATH:
        bouncePath();
        break;
    case AUTONAVSLALOMPATH:
        autoNavSlalomPath();
        break;
    case PATH_B:
        pathB();
        break;
    case BARREL_RACING_PATH:
        barrelRacingPath();
        break;
    }
}

void Autonomous::reset(){
    step = 0;
    trenchStep = -1;

    // Galactic Search Path A Values
    pathAEnd = false;
    pathANumberOfCells = 0;
    pathAPath = 0;
    pathAAction = 0;
    pathAValue = 0;
    pathADrove = false;
    pathATimerOn = false;

    // Autonav Bounce Path Values
    bouncePathEnd = false;
    bouncePathAction = 0;
    bouncePathValue = 0;

    //path B values
    pathBAction = 0;
    pathBValue = 0;
    intakeValue = 0;
    pathBMode = 0;
}

void Autonomous::sendModesToDashboard()
{
    char buffer[256] = "";

    handleDashboardString(DONOTHING, "doing nothing", buffer); //do nothing
    handleDashboardString(CROSS_PORT, "start aligned with prt, drive, shoot", buffer); //start aligned with port, drive, shoot
    handleDashboardString(DRIVE_RIGHT_MORE, "start left wall, drive, shoot", buffer); //start left wall, drive, shoot
    handleDashboardString(SHOOT_FROM_LINE, "start aligned with prt, shoot, drive to CTR", buffer); //start aligned with port, shoot, drive to CTR
    handleDashboardString(JUST_LINE, "drive to ctr", buffer); //drive to CTR
    handleDashboardString(DRIVE_LEFT, "start right wall, drive, shoot", buffer); //start right wall, drive, shoot
    // handleDashboardString(CROSS_PORT_TRENCH, "start aligned with prt, drive, shoot, pick trench balls", buffer); //start aligned with prt, drive, shoot, pick trench balls
    handleDashboardString(FIVE_BALL_AUTO, "start right wall, pick up balls and shoot", buffer); //start right wall, pick up balls and shoot
    handleDashboardString(PATH_A, "galactic search path a", buffer);
    handleDashboardString(PATH_B, "galactic search path b", buffer);
    handleDashboardString(BOUNCE_PATH, "autonav bounce path", buffer);
    handleDashboardString(AUTONAVSLALOMPATH, "AutoNav for Slalom path", buffer);
    handleDashboardString(BARREL_RACING_PATH, "AutoNav Barrel Racing Path", buffer);
    SmartDashboard::PutString("thunderdashboard_auto_list", buffer);
}

void Autonomous::getModeFromDashboard()
{
    int mode = SmartDashboard::GetNumber("Auto_Mode", 0);
    currentAutoMode = (AutoMode)mode;
}

void Autonomous::debug(Feedback *feedback)
{
    feedback->sendDouble("Auto", "step", step);
    feedback->sendDouble("Auto", "current_AutoMode", currentAutoMode);
    

}

void Autonomous::doNothing()
{
    // if it does nothing is it doing something or nothing?
    // don't actually do anything
}

void Autonomous::justLine(){
    if (step == 0)
    {
        drive->autoForward(3 * 12, kMotorSpeed);
        step++;
    }
    else if (step == 1)
    {
        if (drive->autoComplete())
        {
            step++;
        }
    }
}

void Autonomous::crossLine()
{
    if (step == 0)
    {
        drive->autoForward(-105.5, kMotorSpeed);
        step++;
    }
    else if (step == 1)
    {
        if (drive->autoComplete())
        {
            step++;
        }
    }
    else if(step == 2){
        gamePiece->hood(Shooter::HoodPosition::GREEN);
        step++;
        timer.Reset();
        timer.Start();
    }
    else if(step == 3){
        if(timer.Get() >= 1){
            step++;
        }
    }
    else if (step == 4)
    {
        gamePiece->activateShooter(true);
        step++;
    }
    else if (step == 5)
    {
        if(gamePiece->getCellCount() == 0){
        step++;
        trenchStep = 0;
        }
    }
}

void Autonomous::driveRight()
//i believe the distance from the center of the power port is 58" vertical, 75" horizontal. this is 1/2 the real field dimensions.
{
    if (step == 0)
    {
        drive->autoForward(-3, kMotorSpeed);
        step++;
    }
    else if (step == 1)
    {
        if (drive->autoComplete())
        {
            step++;
        }
    }
    else if (step == 2)
    {
        drive->autoPivot(-28, kMotorSpeed);
        step++;
    }
    else if (step == 3)
    {
        if (drive->autoComplete())
        {
            step++;
        }
    }
    else if (step == 4)
    {
        drive->autoForward(-180, kMotorSpeed);
        step++;
    }
    else if (step == 5)
    {
        if (drive->autoComplete())
        {
            step++;
        }
    }
    else if (step == 6)
    {
        drive->autoPivot(-80, kMotorSpeed);
        step++;
    }
    else if (step == 7)
    {
        if (drive->autoComplete())
        {
            step++;
        }
    }
    else if (step == 8)
    {
        drive->autoForward(-20, kMotorSpeed);
        step++;
    }
    else if (step == 9)
    {
        if (drive->autoComplete())
        {
            step++;
        }
    }
    else if(step == 10){
        gamePiece->hood(Shooter::HoodPosition::GREEN);
        step++;
        timer.Reset();
        timer.Start();
    }
    else if(step == 11){
        if(timer.Get() >= 1){
            step++;
        }
    }
    else if (step == 12)
    {
        gamePiece->activateShooter(true);
        step++;
    }
    else if (step == 13)
    {
        if(gamePiece->getCellCount() == 0){
        step++;
        }
    }
}

void Autonomous::driveRightMore()
//58" vertical, 104.7"(104") horizontal from the center of the power port. 1/2 dimensions from real field
{
    if (step == 0)
    {
        drive->autoForward(-6, kMotorSpeed);
        step++;
    }
    else if (step == 1)
    {
        if (drive->autoComplete())
        {
            step++;
        }
    }
    else if (step == 2)
    {
        drive->autoPivot(-24.5, kMotorSpeed);
        step++;
    }
    else if (step == 3)
    {
        if (drive->autoComplete())
        {
            step++;
        }
    }
    else if (step == 4)
    {
        drive->autoForward(-176, kMotorSpeed);
        step++;
    }
    else if (step == 5)
    {
        if (drive->autoComplete())
        {
            step++;
        }
    }
    else if (step == 6)
    {
        drive->autoPivot(-80, kMotorSpeed);
        step++;
    }
    else if (step == 7)
    {
        if (drive->autoComplete())
        {
            step++;
        }
    }
    else if (step == 8)
    {
        drive->autoForward(-76, kMotorSpeed);
        step++;
    }
    else if (step == 9)
    {
        if (drive->autoComplete())
        {
            step++;
        }
    }
    else if(step == 10){
        gamePiece->hood(Shooter::HoodPosition::GREEN);
        step++;
        timer.Reset();
        timer.Start();
    }
    else if(step == 11){
        if(timer.Get() >= 1){
            step++;
        }
    }
    else if (step == 12)
    {
        gamePiece->activateShooter(true);
        step++;
    }
    else if (step == 13)
    {
        if(gamePiece->getCellCount() == 0){
        step++;
        }
    }
}
void Autonomous::shoot()
{
    if(step == 0){
        gamePiece->hood(Shooter::HoodPosition::GREEN);
        step++;
        timer.Reset();
        timer.Start();
    }
    else if(step == 1){
        if(timer.Get() >= 1){
            step++;
        }
    }
    if (step == 2)
    {
        gamePiece->activateShooter(true);
        step++;
    }
    else if (step == 3)
    {
        if(gamePiece->getCellCount() == 0){
        step++;
        }
    }
}

void Autonomous::shootLine(){
    if(step == 0){
        gamePiece->hood(Shooter::HoodPosition::YELLOW);
        step++;
        timer.Reset();
        timer.Start();
        gamePiece->setShooterPreheat(true);
    }
    else if(step == 1){
        if(timer.Get() >= 2.5){
            step++;
        }
    }
    else if(step == 2){
        gamePiece->activateShooter(true);
        step++;
    }
    if(step == 3){
        if(gamePiece->getCellCount() == 0){
            gamePiece->activateShooter(false);
            step++;
        }
    }
    else if(step == 4){
        gamePiece->setShooterPreheat(false);
        drive->autoForward(12*4, kMotorSpeed);
        step++;
    }
    else if(step == 5){
        if(drive->autoComplete()){
            step++;
        }
    }
}

void Autonomous::driveLeft()
{
    if(step == 0){
        drive->autoForward(kRightWallForwardZero,kMotorSpeed);
        step++;
    }
    else if(step == 1){
        if(drive->autoComplete()){
            step++;
        }
    }
    else if(step == 2){
        drive->autoPivot(kRightWallTurnTwo, kMotorSpeed);
        step++;
    }
    else if(step == 3){
        if(drive->autoComplete()){
            step++;
        }
    }
    else if(step == 4){
        drive->autoForward(kRightWallForwardFour, kMotorSpeed);
        step++;
    }
    else if(step == 5){
        if(drive->autoComplete()){
            step++;
        }
    }
    else if(step == 6){
        drive->autoPivot(kRightWallTurnSix, kMotorSpeed);
        step++;
    }
    else if(step == 7){
        if(drive->autoComplete()){
            step++;
        }
    }
    else if(step == 8){
        drive->autoForward(kRightWallForwardEight, kMotorSpeed);
        step++;
    }
    else if(step == 9){
        if(drive->autoComplete()){
            step++;
        }
    }
    else if(step == 10){
        gamePiece->hood(Shooter::HoodPosition::YELLOW);
        step++;
        timer.Reset();
        timer.Start();
    }
    else if(step == 11){
        if(timer.Get() >= 1){
            step++;
        }
    }
    else if(step == 12){
        gamePiece->activateShooter(true);
        step++;
    }
    else if(step == 13){
        if(gamePiece->getCellCount() == 0){
            gamePiece->activateShooter(false);
            step++;
        }
    }
}
void Autonomous::trenchBalls()
{
    if(trenchStep == 0){
        drive->autoForward(kTrenchForwardZero, kMotorSpeed);
        trenchStep++;
    }
    else if(trenchStep == 1){
        if(drive->autoComplete()){
            trenchStep++;
        }
    }
    else if(trenchStep == 2){
        drive->autoPivot(kTrenchTurnTwo, kMotorSpeed);
        trenchStep++;
    }
    else if(trenchStep == 3){
        if(drive->autoComplete()){
            trenchStep++;
        }
    }
    else if(trenchStep == 4){
        drive->autoForward(kTrenchForwardFour, kMotorSpeed);
        trenchStep++;
    }
    else if(trenchStep == 5){
        if(drive->autoComplete()){
            trenchStep++;
        }
    }
    else if(trenchStep == 6){
        drive->autoPivot(kTrenchTurnSix, kMotorSpeed);
        trenchStep++;
    }
    else if(trenchStep == 7){
        if(drive->autoComplete()){
            trenchStep++;
        }
    }
    else if(trenchStep == 8){
        gamePiece->setIntakeState(GamePiece::INTAKE);
        trenchStep++;
    }
    else if(trenchStep == 9){
        drive->autoForward(216, kMotorSpeed);
        trenchStep++;
    }
    else if(trenchStep == 10){
        if(drive->autoComplete()){
            trenchStep++;
        }
    }
    else if(trenchStep == 11){
        gamePiece->setIntakeState(GamePiece::NOTTAKE);
        trenchStep++;
    }
}

void Autonomous::fiveBallAuto()
{
    
    if(step == 0){
        gamePiece->setIntakeState(GamePiece::INTAKE);
        step++;
        timer.Reset();
        timer.Start();
    }
    else if(step == 1){
        if(timer.Get() >= 1){
            step++;
        }
    }
    else if(step == 2){
        drive->autoForward(kFBAForwardTwo, kMotorSpeed);
        step++;
    }
    else if(step == 3){
        if(drive->autoComplete()){
            step++;
        }
    }
    else if(step == 4){
        gamePiece->setIntakeState(GamePiece::NOTTAKE);
        step++;
        timer.Reset();
        timer.Start();
    }
    else if(step == 5){
        if(timer.Get() >= .5){
            step++;
        }
    }
    else if(step == 6){
        drive->autoForward(kFBAForwardEight, kMotorSpeed);
        step++;
    }
    else if(step == 7){
        if(drive->autoComplete()){
            step++;
        }
    }
    else if(step == 8){
        timer.Reset();
        timer.Start();
        step++;
        //autoAim();
    }
    else if(step == 9){
        if(timer.Get() >= 1){
            step++;
        }
    }
    else if (step == 10)
    {
        gamePiece->activateShooter(true);
        step++;
    }
    else if (step == 11)
    {
        if(gamePiece->getCellCount() == 0){
            step++;
            gamePiece->activateShooter(false);
        }
    }
}

void Autonomous::handleDashboardString(AutoMode mode, const char *description, char *buffer)
{
    char key[32];

    sprintf(&buffer[strlen(buffer)], ",%d", mode);
    sprintf(key, "thunderdashboard_auto_%d", mode);
    SmartDashboard::PutString(key, description);
}

void Autonomous::doStartupThings()
{
    gamePiece->hood(Shooter::GREEN);
    getModeFromDashboard();
}




/**
 * I have implemented this new way of doing
 * autonomous that is less repetative and
 * easier to use
 * 
 * To understand how it works, go down and
 * look at the bouncePath() function first
 * b/c it has less steps than pathA() and 
 * is simpler.
 */

// Common Actions:
#define DRIVE 1
#define PIVOT 2
#define START 3
#define END   4

// Path A/B-Specific Actions:
#define IS_CELL 5
#define CHECK_INTAKE_POS 6

/**
 * Galactic Search Path A
 * Robot starts at (30", 90") facing the End Zone
 * 
 * Robot goes forward and checks if there is a ball,
 * then runs the rest of the path accordingly
 */

// Path A
void Autonomous::pathA() {
    // Constant speeds
    static constexpr float pathADriveSpeed { 0.3 };
    static constexpr float pathAPivotSpeed { 0.3 };

    // Steps before we've checked for the 1st Power Cell
    static constexpr int initActions[]  = { START, DRIVE, CHECK_INTAKE_POS, IS_CELL }; // What the robot should do
    static constexpr float initValues[] = {   0,    60,          0,            0    }; // How much the robot should do it

    // Steps if we picked up a ball
    static constexpr int path1Actions[]  = { PIVOT, DRIVE, PIVOT, DRIVE,  PIVOT, DRIVE, END }; // What the robot should do
    //FASTER static constexpr float path1Values[] = { 39.5, 61,     -96,   94.87,   71.57,  180,   0 }; // How much the robot should do it
    static constexpr float path1Values[] = { 33,     67.08,-92, 94.87,  67,  180,   0 }; // How much the robot should do it
    // Steps if we didn't pick up a ball
    static constexpr int path2Actions[]  = { PIVOT, DRIVE, IS_CELL, PIVOT, DRIVE, IS_CELL, PIVOT, DRIVE, IS_CELL, PIVOT, DRIVE, END }; // What the robot should do
    static constexpr float path2Values[] = { 40, 108.17,   0,   -100,    100,   0,       97  , 67.08,    0,    -35,  90,    0 }; // How much the robot should do it

    // To tell how long we are waiting to intake a ball
    static frc::Timer intakeTimer {};
    static constexpr float intakeTimeout { 1.00 }; // The maximum amount of seconds we'll wait for a ball


    if((drive->autoComplete() && !pathAEnd) || !pathADrove) {
        // Sets the current action & value to the action & value for the current step depending on the current path
        if(!pathAPath) { // No path decided yet
            pathAAction = initActions[step];
            pathAValue = initValues[step];
        }
        else if(pathAPath == 1) { // If there is a ball
            pathAAction = path1Actions[step];
            pathAValue = path1Values[step];
        }
        else if(pathAPath == 2) { // If there isn't a ball
            pathAAction = path2Actions[step];
            pathAValue = path2Values[step];
        }

        // The Actions
        if(pathAAction == START) {
            gamePiece->setIntakeState(GamePiece::INTAKE); // Turns Intake On
            pathANumberOfCells = gamePiece->getCellCount(); // Initializes the cell count
            step++;
        }
        else if(pathAAction == DRIVE) {
            drive->autoForward(pathAValue, pathADriveSpeed); // Drive the amount specified in the value for this action
            pathADrove = true;
            step++;
        }
        else if(pathAAction == PIVOT) {
            drive->autoPivot(pathAValue, pathAPivotSpeed); // Pivot the amount specified in the value for this action
            pathADrove = true;
            step++;
        }
        else if(pathAAction == CHECK_INTAKE_POS) {
            if(gamePiece->isIntakeAtPosition()) {
                step++;
            }
        }
        // Checks if we intaked a Power Cell within the specified time in intakeTimeout
        else if(pathAAction == IS_CELL) {
            if(!pathATimerOn) { // Run only the fist time this function is called
                // Starts the intake timer
                intakeTimer.Reset();
                intakeTimer.Start();
                pathATimerOn = true;
            }
            else { // Run the rest of the times the function is called
                if(pathANumberOfCells != gamePiece->getCellCount()) { // If we intaked a power cell
                    pathANumberOfCells = gamePiece->getCellCount(); // Update the cell count
                    if(!pathAPath) { // If we haven't chosen a path yet
                        step = 0;
                        pathAPath = 1; // We picked up a ball, so we'll take path 1
                    } else{
                        step++;
                    }
                    
                    intakeTimer.Stop();
                    pathATimerOn = false;
                }
                else if(intakeTimer.Get() >= intakeTimeout) { // If we've exceeded the timeout
                    if(!pathAPath) { // If we haven't chosen a path yet
                        step = 0;
                        pathAPath = 2; // We didn't pick up a ball, so we'll take path 2
                    }
                    else {
                        // We didn't pick up a ball when we were supposed to,
                        // so we'll move to the last step to stop the robot
                        pathAAction = END;
                    }
                    intakeTimer.Stop();
                    pathATimerOn = false;
                }
            }
        }
        if(pathAAction == END) { // Not `else if` b/c something ^ may want to stop the robot
            gamePiece->setIntakeState(GamePiece::NOTTAKE); // Turn intake off
            pathAEnd = true; // Stops the initial if statement from going again
        }
    }
}

/**
 * Autonav Bounce Path
 * Robot starts at (30", 90") facing the finish
 */
void Autonomous::bouncePath() {
    // Constant Speeds
    static constexpr float bouncePathDriveSpeed { 0.6 };
    static constexpr float bouncePathPivotSpeed { 0.3 };

    // Steps for the Robot
    static constexpr int actions[]  = { DRIVE, PIVOT, DRIVE, PIVOT, DRIVE, PIVOT, DRIVE, PIVOT, DRIVE, PIVOT/*added pivot here because there was kickback from switching from forward to backward*/, DRIVE, PIVOT, DRIVE, PIVOT, DRIVE, PIVOT, DRIVE, PIVOT, DRIVE, END }; // What the robot should do
    static constexpr float values[] = {  44,   -85,    38,    -15,  -104,   90,     41,    -72,    86,   3,                                                                                          -97,   80,    65.5,    -76,   95,     1,   -40,     -50,   -18,    0 }; // How much the robot should do it
    
    if(step == 0 || (drive->autoComplete() && !bouncePathEnd)) {
        // Sets the current action & value to the action & value for the current step
        bouncePathAction = actions[step];
        bouncePathValue = values[step];

        // The Actions
        if(bouncePathAction == DRIVE) {
            drive->autoForward(bouncePathValue, bouncePathDriveSpeed); // Drive the amount specified in the value for this action
            step++;
        }
        else if(bouncePathAction == PIVOT) {
            drive->autoPivot(bouncePathValue, bouncePathPivotSpeed); // Pivot the amount specified in the value for this action
            step++;
        }
        else if(bouncePathAction == END) {
            bouncePathEnd = true; // Stops the initial if statement from going again
        }
    }
}
void Autonomous::autoNavSlalomPath(){
    static constexpr float slalomPathDriveSpeed { 0.7 };
    static constexpr float slalomPathPivotSpeed { 0.4 };


    static constexpr int modes[] = { DRIVE, PIVOT, DRIVE, PIVOT, DRIVE, PIVOT, DRIVE, PIVOT, DRIVE, PIVOT, DRIVE, PIVOT, DRIVE, PIVOT, DRIVE, PIVOT, DRIVE, PIVOT, DRIVE, PIVOT, DRIVE, END};
    static constexpr float values[] = { 35, -70,    30,     72,   143,    75,    44,   -71,   43,    -71,   43,   -69,     40,   -69,   43,    76,   143,   75,     65,   -80,   55,    0 };

    int mode {};
    float value {};

    if (step == 0 || drive -> autoComplete()){
        mode = modes[step];
        value = values[step];
        if(mode == DRIVE){
            drive->autoForward(value, slalomPathDriveSpeed);
            step++;
        }
        else if(mode == PIVOT){
            drive ->autoPivot(value, slalomPathPivotSpeed);
            step++;
        }
        else if(mode == END){
           // do nothing 
        }
        
    }
}


void Autonomous::barrelRacingPath(){
    static constexpr float barrelRacingDriveSpeed { 0.6 };
    static constexpr float barrelRacingPivotSpeed { 0.6 };
 
 
    static constexpr int modes[] = { DRIVE, PIVOT, DRIVE, PIVOT, DRIVE, PIVOT, DRIVE, PIVOT, DRIVE, PIVOT, DRIVE, PIVOT, DRIVE, PIVOT, DRIVE, PIVOT, DRIVE, PIVOT, DRIVE, PIVOT, DRIVE, END};
    static constexpr float values[] = { 110, 60,    54,    60,   48,     60,    48,     60,    143,  -60,    39,   -57,   50,   -50,     33,   -50,    154,   -57,   104,  -82,   320,    0};
 
    int mode {};
    float value {};
 
    if (step == 0 || drive -> autoComplete()){
        mode = modes[step];
        value = values[step];
        if(mode == DRIVE){
            drive->autoForward(value, barrelRacingDriveSpeed);
            step++;
        }
        else if(mode == PIVOT){
            drive ->autoPivot(value, barrelRacingPivotSpeed);
            step++;
        }
        else if(mode == END){
           // do nothing 
        }
    }
}


// Path B
void Autonomous::pathB(){
    static constexpr float pathBDriveSpeed{0.3};
    static constexpr float pathBPivotSpeed{0.3};

    static constexpr int initActions[] = {START, DRIVE, IS_CELL};
    static constexpr float initValues[] = {0, 54, 0};

    static constexpr int isBallActions[] = {PIVOT, DRIVE, PIVOT, DRIVE, PIVOT, DRIVE, END};
    static constexpr float isBallValues[] = {90, 84.85, -97.5, 105, 50, 150, 0};

    static constexpr int noBallActions[] = {PIVOT, DRIVE, PIVOT, DRIVE, PIVOT, DRIVE, END};
    static constexpr float noBallValues[] = {55,    84,    -70,    90,   52,  148, 0};
    if (pathBMode == 0)
    {
        pathBAction = initActions[step];
        pathBValue = initValues[step];
    }
    else if (pathBMode == 1)
    {
        pathBAction = noBallActions[step];
        pathBValue = noBallValues[step];
    }
    else if (pathBMode == 2)
    {
        pathBAction = isBallActions[step];
        pathBValue = isBallValues[step];
    }
    if(drive->autoComplete() || pathBAction == START){
    if (pathBAction == START)
    {
        drive->autoForward(0, kMotorSpeed);
        gamePiece->setIntakeState(GamePiece::INTAKE);
        step++;
    }
    else if (pathBAction == DRIVE && drive->autoComplete())
    {
        drive->autoForward(pathBValue, pathBDriveSpeed);
        step++;
    }
    else if (pathBAction == PIVOT)
    {
        drive->autoPivot(pathBValue, pathBPivotSpeed);
        step++;
    }
    else if (pathBAction == END)
    {
        gamePiece->setIntakeState(GamePiece::NOTTAKE);
    }
    else if (pathBAction == IS_CELL)
    {
        if (gamePiece->getCellCount() == 0)
        {
            pathBMode = 1;
            step = 0;
        }
        else if(gamePiece->getCellCount() == 1)
        {
            pathBMode = 2;
            step = 0;
        }
    }
    }
}