#include "ControlPanel.h"
#include <math.h>
#include <wpi/timestamp.h>

const double kRotationControlSpeed = .5;

ControlPanel::ControlPanel() : spinWheelMotor(ThunderSparkMax::create(ThunderSparkMax::MotorID::WheelOfFortune))
{
}

void ControlPanel::reset()
{
    state = ControlPanelState::STOPPED;
    PositionControlColor = DONT_KNOW;
    lastColorDetected = DONT_KNOW;
    sameColorCounted = 0;
    lastCommandFinished = false;
    sensorWasInRange = false;
    lastProxValue = 0;
    lastSensorRed = lastSensorGreen = lastSensorBlue = 0;
    spinWheelMotor->Set(0);
}

ControlPanel::ControlPanelColor ControlPanel::senseColor()
{
#ifdef TEST_BOARD
    frc::Color detectedColor = frc::Color(0, 0, 0);
#else
    frc::Color detectedColor = m_colorSensor.GetColor();
#endif


    double red = detectedColor.red; // how much red in the colour we see
    double green = detectedColor.green; // same as da red one but green
    double blue = detectedColor.blue; // same as da red but blue
    lastSensorRed = red;
    lastSensorGreen = green;
    lastSensorBlue = blue;
    // method to determine what color it is
    if (red >= .10 && red <= .25 && green >= .35 && green <= .53 && blue >= .25 && blue <= .53)
    {
        return BLUE;
    }
    else if (red >= .25 && red <= .55 && green >= .3 && green <= .5 && blue >= .1 && blue <= .3)
    {
        return RED;
    }
    else if (red >= .05 && red <= .25 && green >= .45 && green <= .65 && blue >= .15 && blue <= .30)
    {
        return GREEN;
    }
    else if (red >= .22 && red <= .37 && green >= .45 && green <= .59 && blue >= .05 && blue <= .15)
    {
        return YELOW;
    }
    else
    {
        return DONT_KNOW;
    }
}

void ControlPanel::process()
{
    double motorSpeed = 0;
    std::string gameData;
    ControlPanelColor DetectedColor = senseColor();

    // Counts every time it sees red
    if (DetectedColor != lastColorDetected)
    {
        if (DetectedColor == RED && state == ROTATION)
        {
            sameColorCounted++;
        }
    }

    if (DetectedColor != DONT_KNOW)
        // Don't save "don't know" because we don't want stray "don't knows"
        // to increment our rotation count
        lastColorDetected = DetectedColor;

    // Raw color regardless of don't know for debug
    lastRawColorDetected = DetectedColor;

    // Activates rotation control if all conditions are met
    if (state == ControlPanelState::ROTATION && rotationControlDone() == false && overWheel() == true)
    {
        motorSpeed = kRotationControlSpeed;
    }
    else if (overWheel() == true && state == POSITION)
    {
        if(DetectedColor == GREEN && PositionControlColor == RED)
        {
            action = NOTRED;    
            motorSpeed = -.2;
        }
        if(DetectedColor == RED && PositionControlColor == RED)
        {
            lastCommandFinished = true;
            motorSpeed = 0;
        }
        else if(DetectedColor != RED && PositionControlColor == RED)
        {
            motorSpeed = .2;
            lastCommandFinished = false;
        }
        if(DetectedColor == RED && PositionControlColor == YELOW)
        {
            action = NOTYELOW;
            motorSpeed = -.2;
        }
        if(DetectedColor == YELOW && PositionControlColor == YELOW)
        {
            lastCommandFinished = true;
            motorSpeed = 0;
        }
        else if(DetectedColor != YELOW && PositionControlColor == YELOW)
        {
            motorSpeed = .2;
            lastCommandFinished = false;
        }
        if(DetectedColor == YELOW && PositionControlColor == BLUE)
        {
            action = NOTBLUE;
            motorSpeed = -.2;
        }
        if(DetectedColor == BLUE && PositionControlColor == BLUE)
        {
            lastCommandFinished = true;
            motorSpeed = 0;
        }
        else if(DetectedColor != BLUE && PositionControlColor == BLUE)
        {
            motorSpeed = .2;
            lastCommandFinished = false;
        }
        if(DetectedColor == BLUE && PositionControlColor == GREEN)
        {
            action = NOTGREEN;
            motorSpeed = -.2;
        }
        if(DetectedColor == GREEN && PositionControlColor == GREEN)
        {
            lastCommandFinished = true;
            motorSpeed = 0;
        }
        else if(DetectedColor != GREEN && PositionControlColor == GREEN)
        {
            motorSpeed = .2;
            lastCommandFinished = false;
        }
    }
    if (state == MANUALFAST)
    {
        motorSpeed = .5;
    }
    if (state == MANUALSLOW)
    {
        motorSpeed = .25;
    }
    spinWheelMotor->Set(motorSpeed);
}

const char *ControlPanel::colorToString(ControlPanelColor color)
{
    const char *thestring = "";
    if (color == RED)
    {
        thestring = "Red";
    }
    else if (color == BLUE)
    {
        thestring = "Blue";
    }
    else if (color == GREEN)
    {
        thestring = "Green";
    }
    else if (color == YELOW)
    {
        thestring = "Yelow";
    }
    else if (color == DONT_KNOW)
    {
        thestring = "?";
    }
    return thestring;
}

void ControlPanel::debug(Feedback *feedback)
{
    const char *stateString = "";
    switch (state)
    {
    case POSITION:
        stateString = "Postion";
        break;
    case ROTATION:
        stateString = "Rotation";
        break;
    case STOPPED:
        stateString = "Stopped";
        break;
    case MANUALFAST:
        stateString = "ManualFast";
        break;
    case MANUALSLOW:
        stateString = "ManualSlow";
        break;
    }
    senseColor();
    overWheel();
    feedback->sendString("controlPanel", "senseColor", colorToString(senseColor()));
    feedback->sendString("controlPanel", "positionColor", colorToString(PositionControlColor));
    feedback->sendDouble("controlPanel", "rotationCount", sameColorCounted);
    feedback->sendBoolean("controlPanel", "proxInRange", sensorWasInRange);
    feedback->sendDouble("controlPanel", "proxValue", lastProxValue);
    feedback->sendDouble("controlPanel", "colorRed", lastSensorRed);
    feedback->sendDouble("controlPanel", "colorGreen", lastSensorGreen);
    feedback->sendDouble("controlPanel", "colorBlue", lastSensorBlue);
    feedback->sendString("controlPanel", "state", stateString);

    std::string gameData = frc::DriverStation::GetInstance().GetGameSpecificMessage();
    ControlPanelColor driverTargetColor = DONT_KNOW;
        if (gameData.length() > 0)
        {
            // adjust position control color variable to what game sensor will see
            switch (gameData[0])
            {
            case 'B':
                driverTargetColor = RED;
                break;
            case 'G':
                driverTargetColor = YELOW;
                break;
            case 'R':
                driverTargetColor = BLUE;
                break;
            case 'Y':
                driverTargetColor = GREEN;
                break;
            default:
                driverTargetColor = DONT_KNOW;
                break;
            }
        }
        else
        {
            driverTargetColor = DONT_KNOW;
        }

            int dashboardColorValue = 0;
    switch(senseColor()) {
        case RED:
        dashboardColorValue = 1;
        break;
        case YELOW:
        dashboardColorValue = 4;
        break;
        case BLUE:
        dashboardColorValue = 3;
        break;
        case GREEN:
        dashboardColorValue = 2;
        break;
        case DONT_KNOW:
        dashboardColorValue = 0;
        break;
    }

    if(!overWheel()) dashboardColorValue = 0;
    feedback->sendDouble("thunderdashboard", "wheel", dashboardColorValue);

    int dashboardPositionCtlValue = 0;
    switch(driverTargetColor) {
        case RED:
        dashboardPositionCtlValue = 1;
        break;
        case YELOW:
        dashboardPositionCtlValue = 4;
        break;
        case BLUE:
        dashboardPositionCtlValue = 3;
        break;
        case GREEN:
        dashboardPositionCtlValue = 2;
        break;
        case DONT_KNOW:
        dashboardPositionCtlValue = 0;
        break;
    }
    feedback->sendDouble("thunderdashboard", "wheel_end", dashboardPositionCtlValue);
    
}

void ControlPanel::positionControl()
{
    if (state != POSITION && overWheel())
    {
        std::string gameData;
        lastCommandFinished = false;
        gameData = frc::DriverStation::GetInstance().GetGameSpecificMessage();
        printf("Game data is {%s}", gameData.c_str());

        state = ControlPanelState::POSITION;
        if (gameData.length() > 0)
        {
            // adjust position control color variable to what game sensor will see
            switch (gameData[0])
            {
            case 'B':
                PositionControlColor = RED;
                break;
            case 'G':
                PositionControlColor = YELOW;
                break;
            case 'R':
                PositionControlColor = BLUE;
                break;
            case 'Y':
                PositionControlColor = GREEN;
                break;
            default:
                PositionControlColor = DONT_KNOW;
                break;
            }
        }
        else
        {
            PositionControlColor = DONT_KNOW;
        }
    }
}

void ControlPanel::rotationControl()
{
    // if it is not already in rotation mode while over control panel then make it rotation mode
    if (state != ROTATION)
    {
        if (overWheel() == true)
        {
            lastCommandFinished = false;
            lastColorDetected = RED;
            sameColorCounted = 0;
            state = ROTATION;
        }
    }
}

void ControlPanel::stop()
{
    state = STOPPED;
    lastCommandFinished = false;
    sameColorCounted = 0;
}

bool ControlPanel::overWheel()
{
    // makes sure the sensor is actually over the control panel
#ifdef TEST_BOARD
    int proxValue = 0;
#else
    int proxValue = m_colorSensor.GetProximity();
#endif
    lastProxValue = proxValue;
    if (proxValue > 200)
    {
        if (!sensorWasInRange)
            printf("ControlPanel - sensor in range %d\n", proxValue);
        sensorWasInRange = true;
        return true;
    }
    else
    {
        if (sensorWasInRange)
            printf("ControlPanel - sensor out range %d\n", proxValue);
        sensorWasInRange = false;
        return false;
    }
}
bool ControlPanel::rotationControlDone()
{
    // counts how many times the colour red passes
    if (sameColorCounted >= 6)
    {
        lastCommandFinished = true;
        return true;
    }
    else
    {
        return false;
    }
}
void ControlPanel::manualDriveFast()
{
    state = MANUALFAST;
    lastCommandFinished = false;
}

void ControlPanel::manualDriveSlow()
{
    state = MANUALSLOW;
    lastCommandFinished = false;
}

bool ControlPanel::hasFinished()
{
    return lastCommandFinished;
}

void ControlPanel::doLights(Lights *lights)
{
    if (sensorWasInRange) {
        switch (lastRawColorDetected) {
            case RED:
                lights->setControlPanelColor(true, 1, 0, 0);
                break;
            case GREEN:
                lights->setControlPanelColor(true, 0, 1, 0);
                break;
            case BLUE:
                lights->setControlPanelColor(true, 0, 0, 1);
                break;
            case YELOW:
                lights->setControlPanelColor(true, 1, 1, 0);
                break;
            default:
                lights->setControlPanelColor(true, 0, 1, 1);
                break;
        }
    } else
        lights->setControlPanelColor(false, 0, 0, 0);

}
