#include "Controls.h"

// Drive Control Bindings
const int kDriveToggleModes = 3; // x button
const int kDriveSwapCamera = 2; // B button
const int kDriveLimelight = 1; // a button

const int kDriveTankLeftAxis = 1; // left stick, y axis
const int kDriveTankRightAxis = 5; // right stick, y axis

const int kDriveArcadeForwardAxis = 1; // left stick, y axis
const int kDriveArcadeTurnAxis = 4; // right stick, x axis

const int kDriveLeftTurboAxis = 2; // left trigger
const int kDriveRightTurboAxis = 3; // right trigger

const int kDriveLeftSlow = 5; // left bumper
const int kDriveRightSlow = 6; //right bumper

const int kCalibrateGyro = 4; // Y button

// Aux Control Bindings
const int kAuxControlPanelSpin = 2;

// Tunable constants
const double kSlowMultiplier = .3; // .3
const double kNormalMultiplier = .6; // .5
const double kTurboMultiplier = .7; // .7
const double kMaxDriveSpeed = .8;
const double kTurboPreRampDuration = 1; // wait 1 second after starting turbo to speed it up more

const double kTurnMultiplier = .65;

const double kArcadeTurnDeadzone = .1;
const double kTankDeadzone = .1;

// Xbox button maps
const int kAButton = 1; // GetRawButton() give bool
const int kBButton = 2; // GetRawButton() give bool
const int kXButton = 3; // GetRawButton() give bool
const int kYButton = 4; // GetRawButton() give bool
const int kLeftBumper = 5; // GetRawButton() give bool
const int kRightBumper = 6; // GetRawButton() give bool
const int kBackButton = 7; // GetRawButton() give bool
const int kStartButton = 8; // GetRawButton() give bool
const int kLeftTrigger = 2; // GetRawAxis() give float
const int kRightTrigger = 3; // GetRawAxis() give float
const int kLeftDial = 1; // GetRawAxis() give float
const int kRightDial = 5; // GetRawAxis() give float
const int kDPad = 0; // GetPOV() give float
// Switches
const int kSwitchOne = 1; // GetRawButton() give bool
const int kSwitchTwo = 2; // GetRawButton() give bool
const int kSwitchThree = 3; // GetRawButton() give bool
const int kSwitchFour = 4; // GetRawButton() give bool
const int kSwitchFive = 5; // GetRawButton() give bool
const int kSwitchSix = 6; // GetRawButton() give bool
const int kSwitchSeven = 7; // GetRawButton() give bool
const int kSwitchEight = 8; // GetRawButton() give bool
const int kSwitchNine = 9; // GetRawButton() give bool

Controls::Controls(Drive* drive, GamePiece* gamePiece, Hang* hang, ControlPanel* controlPanel) {
    this->drive = drive;
    this->gamePiece = gamePiece;
    this->hang = hang;
    this->controlPanel = controlPanel;
}

void Controls::process() {
    processDriver();
    processAux();
    brokenSensor();
}

void Controls::processDriver() {
    bool toggleDriveMode = controllerDriver.GetRawButton(kDriveToggleModes);

    bool limelightTurning = controllerDriver.GetRawButton(kDriveLimelight);

    double tankLeft = -controllerDriver.GetRawAxis(kDriveTankLeftAxis);
    double tankRight = -controllerDriver.GetRawAxis(kDriveTankRightAxis);

    double arcadeForward = -controllerDriver.GetRawAxis(kDriveArcadeForwardAxis);
    double arcadeTurn = controllerDriver.GetRawAxis(kDriveArcadeTurnAxis);

    bool turboLeft = controllerDriver.GetRawAxis(kDriveLeftTurboAxis) > .75;
    bool turboRight = controllerDriver.GetRawAxis(kDriveRightTurboAxis) > .75;

    bool slowLeft = controllerDriver.GetRawButton(kDriveLeftSlow);
    bool slowRight = controllerDriver.GetRawButton(kDriveRightSlow);

    bool calibrateGyro = controllerDriver.GetRawButton(kCalibrateGyro);

    double hangSlowForward = controllerDriver.GetPOV();

    bool swapCamera = controllerDriver.GetRawButtonPressed(kDriveSwapCamera);

    /*switch(limelightState){
        double angle;
        case AIMED:
            break;
        case TURNING:
            angle = limelight->angleHorizontal();
            if(angle >= 3){
                drive->setSpeed(.1, -.1);
            }
        else if(angle <= -3){
            drive->setSpeed(-.1, .1);
        }
        else {
            drive->setSpeed(0, 0);
            limelightState = AIMED;
        }
            break;
        case DRIVING:
            break;
    }*/
    
   /* if(limelightTurning == true){
        if(limelightState == AIMED){
            limelightState = TURNING;
        }
     }*/

    if(calibrateGyro) {
        drive->calibrateGyro();
    }

    if(swapCamera) {
        viewingLimelight = !viewingLimelight;
    }

    if(toggleDriveMode == true) {
        if(wasDriveModeToggled == false) {
            isTankMode = !isTankMode;
        }
    }

    wasDriveModeToggled = toggleDriveMode;

    double finalLeft = 0;
    double finalRight = 0;
    
    if(isTankMode) { // tank drive
        double tankOutputLeft = getPower(tankLeft, turboLeft, slowLeft);
        double tankOutputRight = getPower(tankRight, turboRight, slowRight);

        finalLeft = tankOutputLeft;
        finalRight = tankOutputRight;
    }
    else { // arcade drive
        double arcadeOutputLeft = arcadeForward;
        double arcadeOutputRight = arcadeForward;

        if(arcadeForward < -.05) {
            arcadeTurn *= -1;
        }
        
        if(arcadeTurn < -kArcadeTurnDeadzone) { /// turn left
            if(arcadeOutputLeft > 0) {// if driving forwards
                arcadeOutputLeft -= fabs(arcadeTurn) * kTurnMultiplier;
            }
            else if(arcadeOutputLeft < 0) { // if driving backwards
                arcadeOutputLeft += fabs(arcadeTurn) * kTurnMultiplier;
            }
        }
        else if(arcadeTurn > kArcadeTurnDeadzone) { // turn right
            if(arcadeOutputRight > 0) {// if driving forwards
                arcadeOutputRight -= arcadeTurn * kTurnMultiplier;
            }
            else if(arcadeOutputRight < 0) { // if driving backwards
                arcadeOutputRight += arcadeTurn * kTurnMultiplier;
            }
        }
        
        if(abs(arcadeTurn) > kArcadeTurnDeadzone) {
            if(abs(arcadeForward) < .1) {
                arcadeOutputLeft = arcadeTurn;
                arcadeOutputRight = -arcadeTurn;
            }
        }

        if(turboLeft && turboRight) {
            turboHeld.Start();
        }
        else {
            turboHeld.Reset();
            turboHeld.Stop();
        }

        arcadeOutputLeft = getPower(arcadeOutputLeft, turboLeft, slowLeft);
        arcadeOutputRight = getPower(arcadeOutputRight, turboRight, slowRight);

        finalLeft = arcadeOutputLeft;
        finalRight = arcadeOutputRight;
    }

    if(fabs(finalLeft) < .1 && fabs(finalRight) < .1) {
        if(hangSlowForward == 0) {
            finalLeft = .2;
            finalRight = .2;
        }
    } 

    drive->setSpeed(finalLeft, finalRight);
}


void Controls::processAux() {
    // These are my Aux variables
     bool intakeLT;
     bool shooterHelixRB;
     bool retractHangA;
     bool limelightB;
     bool controlPanelX;
     bool extendHangY;
     float hoodDPad;
     bool outtakeLB;
     float shootBallRT;
     bool hoodGreenZoneSB;
     bool hoodRedZoneBB;
     bool positionRotation;
     bool controlPanelManual;
     double shooterManualSpeed;
     bool hoodManual;
     double hoodSpeed;

     // Sets true if the button is being pressed down
     intakeLT = controllerAux.GetRawAxis(kLeftTrigger) > .4;
     shooterHelixRB = controllerAux.GetRawButton(kRightBumper);
     retractHangA = controllerAux.GetRawButton(kAButton);
     limelightB = controllerAux.GetRawButton(kBButton);
     controlPanelX = controllerAux.GetRawButton(kXButton);
     extendHangY = controllerAux.GetRawButton(kYButton);
     hoodDPad = controllerAux.GetPOV(kDPad);
     outtakeLB = controllerAux.GetRawButton(kLeftBumper);
     shootBallRT = controllerAux.GetRawAxis(kRightTrigger);
     hoodGreenZoneSB = controllerAux.GetRawButton(kStartButton);
     hoodRedZoneBB = controllerAux.GetRawButton(kBackButton);
     shooterManualSpeed = -controllerAux.GetRawAxis(1);
     positionRotation = controllerAux2.GetRawButton(kSwitchSix);
     controlPanelManual = controllerAux2.GetRawButton(kSwitchOne);

     hoodSpeed = -controllerAux.GetRawAxis(5);

     
     if(hoodSpeed > .1 || hoodSpeed < -.1){
         if(hoodSpeed > .1){
             gamePiece->setHoodManualSpeed(((hoodSpeed+.95)*.2)+.3);
         }
         else if(hoodSpeed < -.1){
             gamePiece->setHoodManualSpeed(((hoodSpeed+1.05)*.2)+.3);
         }
     }
    else{
        gamePiece->setHoodManualSpeed(.5);
    }
     // Turns on Intake if Outake is off
     if(intakeLT == true && outtakeLB == false){ // Only runs intake if isnt outtaking
         gamePiece->setIntakeState(GamePiece::INTAKE);
     }
     // Turns on Outtake, turns it off if neither outake or intake are on
     else if(outtakeLB == true && intakeLT == false){ // Only runs outtake if isnt intakking
         gamePiece->setIntakeState(GamePiece::OUTTAKE);
     }
     else{ // Runs neither if both are pressed or if neither are pressed
         gamePiece->setIntakeState(GamePiece::NOTTAKE);
     }
     // Turns on Shooter and Helix if not intaking
     if(shooterHelixRB == true){
         gamePiece->setShooterPreheat(true);
     }
     else{
         gamePiece->setShooterPreheat(false);
     }
     
     // Actually shoots the ball if warmed up
     if(shootBallRT >= .6) {
         gamePiece->activateShooter(true);
     }
     else{
         gamePiece->activateShooter(false);
     }
    /*if(limelightB == true){
        if(limelightState == AIMED){
            limelightState = TURNING;
        }
     }
    switch(limelightState){
        double angle;
        case TURNING:
            angle = limelight->angleHorizontal();
            if(angle >= 3){
                drive->setSpeed(.1, -.1);
            }
        else if(angle <= -3){
            drive->setSpeed(-.1, .1);
        }
        else {
            drive->setSpeed(0, 0);
            limelightState = AIMED;
        }
        break;
        case DRIVING:
            break;
        case AIMED:
            break;

        }
        */

    // Starts control panel if button pressed, if switch is true does rotation, if it isnt does position
    if (controlPanelManual == true){ // Checks if the manual switch is activated, turns into a press and hold
         if(controlPanelX == true){
             if(positionRotation == true){
                 controlPanel->manualDriveFast();
             }
             else{
                 controlPanel->manualDriveSlow();
             }
         }
         else{
             controlPanel->stop();
         }
     }
     else{ // If no manual switch run control panel like normal
         if(controlPanelX == true){ // If the control panel button was pressed
            if(positionRotation == true){ // If the control switch is in rotation mode do rotation control
             controlPanel->rotationControl();
            }
            else{ // If the control switch isnt in rotaition mode do position control
             controlPanel->positionControl();
            }
        }
        else {
            controlPanel->stop();
        }
     }
     if(extendHangY == true && retractHangA == false){ // Extends hang if isnt retracting
         hang->move(Hang::UP);
     }
     // Retracts hang if button pressed
     else if(retractHangA == true && extendHangY == false){ // Retracts hang if isnt extending
         hang->move(Hang::DOWN);
     }
     else{ // If both pressed or neither pressed it does nothing
         hang->move(Hang::STOP);
     }
     // Sets shooter to initiation line
      if(hoodDPad == 0){
        gamePiece->hood(Shooter::GREEN);
     }
     // Sets shooter to Control Panel
     if(hoodDPad == 90){
         gamePiece->hood(Shooter::YELLOW);
     }
     // Sets shooter to wall
     if(hoodDPad == 180){
         gamePiece->hood(Shooter::BLUE);
     }
     if(hoodDPad == 270){
         gamePiece->hood(Shooter::RED);
     }
     

     if(shooterManualSpeed > .1) { // up
        if(desiredShooterSpeed < 1) {
            desiredShooterSpeed += .01;
        }
     }
     else if(shooterManualSpeed < -.1) { // down
        if(desiredShooterSpeed > 0) {
            desiredShooterSpeed -= .01;
        }
     }

     gamePiece->setShooterSpeed(desiredShooterSpeed);
}

double Controls::getPower(double speed, bool turbo, bool slow) {
    if(fabs(speed) <= kTankDeadzone){
            speed = 0;
        }
    speed = speed * speed * speed;
    if(slow) {
        speed *= kSlowMultiplier;
    }
    else if(!turbo) {
        speed *= kNormalMultiplier;
    }
    else {
        speed *= kTurboMultiplier;
        if(speed < kMaxDriveSpeed && turboHeld.Get() > kTurboPreRampDuration) {
            speed += (turboHeld.Get() - kTurboPreRampDuration) * .03;
        }
    }

    return speed;
}
void Controls::brokenSensor() {
    bool intakePivotSensor;
    bool intakeBallCounterSensor;
    bool storageTransitionSensor;
    bool shooterBallCounterSensor;
    bool hangRetractSensor;
    bool shooterManualSpeed;
    bool driveEncBroke;

    intakePivotSensor = controllerAux2.GetRawButton(kSwitchTwo);
    intakeBallCounterSensor = controllerAux2.GetRawButton(kSwitchThree);
    storageTransitionSensor = controllerAux2.GetRawButton(kSwitchFour);
    shooterBallCounterSensor = controllerAux2.GetRawButton(kSwitchFive);
    shooterManualSpeed = controllerAux2.GetRawButton(kSwitchSeven);
    driveEncBroke = controllerAux2.GetRawButton(kSwitchEight);
    hangRetractSensor = controllerAux2.GetRawButton(kSwitchNine);

    drive->setBackdriveBroken(driveEncBroke);
    

    // If each broken switch is flipped it will run a function with true, otherwise it runs said function with false
    if (intakePivotSensor == true){
        gamePiece->pivotSensorBroken(true);
    }
    else{
        gamePiece->pivotSensorBroken(false);
    }
    if (intakeBallCounterSensor == true){
        gamePiece->setIntakeCounterBroken(true);
    }
    else{
        gamePiece->setIntakeCounterBroken(false);
    }
    if (storageTransitionSensor == true){
        gamePiece->setTransitionCounterBroken(true);
    }
    else{
        gamePiece->setTransitionCounterBroken(false);
    }
    if (shooterBallCounterSensor == true){
        gamePiece->setShooterBallCounterBroken(true);
    }
    else{
        gamePiece->setShooterBallCounterBroken(false);
    }
    // if (hangZeroSensor == true){
    //     hang->zeroSensorBroken(true);
    // }
    // else{
    //     hang->zeroSensorBroken(false);
    // }

    if(hangRetractSensor) {
        hang->enableSlowRetract(false);
    }
    else {
        hang->enableSlowRetract(true);
    }

    gamePiece->setShooterManualSpeedEnabled(shooterManualSpeed);
}

void Controls::debug(Feedback* feedback) {
    feedback->sendString("controls", "drive style", isTankMode ? "tank" : "arcade");
    feedback->sendDouble("thunderdashboard", "drivearcade", isTankMode ? 0 : 1);
    feedback->sendDouble("thunderdashboard", "frontcamera", viewingLimelight ? 0 : 1);
}
