#include "Lights.h"
#include "Gamepiece.h"

#include <frc/DriverStation.h>
#include <frc/smartdashboard/SmartDashboard.h>

const int rearOffsets[] = {
    0, // add index
    22, // subtract index
    23, // add index
    57, // subtract index
    58 // add index
};

const int rearLengths[] = {
    11,
    12,
    14,
    21,
    21
};

const int frontOffsets[] = {
    16, // bottom row, subtract index
    17, // add index
    50, // subtract index
    51, // add index
    84 // top row, subtract index
};

const int frontLengths[] = {17, 17, 17, 17, 17};

const int multiplier = 255;

Lights::Lights() {
    strip.SetLength(numLEDTotal);
    strip.SetData(stripBuffer);
    strip.Start();

    timer.Reset();
    timer.Start();
}

void Lights::process() {
    // if(frc::DriverStation::GetInstance().IsEnabled()) {
        for(int i = 0; i < 5; i++) {
            setRow(REAR, i, frc::Color(0, 0, 0));
            setRow(FRONT, i, frc::Color(0, 0, 0));
        }

        if (gyroCalibrating) {
            for (int i = 0; i < 5; i++) {
                if (timer.Get() > 1) {
                    setRow(REAR, i, frc::Color(1, 1, 0));
                    setRow(FRONT, i, frc::Color(1, 1, 0));
                } else if (timer.Get() > 2) {
                    timer.Reset();
                }
                
            }     
        } else if (cpActive) {
            for (int i = 0; i < 5; i++) {
                setRow(REAR, i, frc::Color(cpR, cpG, cpB));
                setRow(FRONT, i, frc::Color(cpR, cpG, cpB));
            }
            
        } else {
            switch (ballCount) {
            case 5:
                for (int i = 0; i < 5; i++) {
                    setRow(FRONT, i, frc::Color(1, 0, 0));
                    setRow(REAR, i, frc::Color(1, 0, 0));
                }
                break;
            case 4:
                setRow(REAR, 3, frc::Color(0, 0, 1));
                setRow(FRONT, 3, frc::Color(0,0,1));
                [[fallthrough]];
            case 3:
                setRow(REAR, 2, frc::Color(0, 1, 0));
                setRow(FRONT, 2, frc::Color(0, 1, 0));
                [[fallthrough]];
            case 2:
                setRow(REAR, 1, frc::Color(1, 1, 0));
                setRow(FRONT, 1, frc::Color(1, 1, 0));
                [[fallthrough]];
            case 1:
                setRow(REAR, 0, frc::Color(1, 0, 0));
                setRow(FRONT, 0, frc::Color(1, 0, 0));
                break;
            case GamePiece::BALL_COUNT_UNKNOWN:
                // 03510
                setPixel(REAR, 0, 2, frc::Color(1, 0,0));
                setPixel(REAR, 1, 4, frc::Color(1, 0,0));
                setPixel(REAR, 2, 6, frc::Color(1, 0,0));
                setPixel(REAR, 3, 9, frc::Color(1, 0,0));
                setPixel(REAR, 4, 8, frc::Color(1, 0, 0));
                setPixel(REAR, 0, 9, frc::Color(1, 0,0));
                setPixel(REAR, 1, 8, frc::Color(1, 0,0));
                setPixel(REAR, 2, 7, frc::Color(1, 0,0));
                setPixel(REAR, 3, 12, frc::Color(1, 0,0));
                setPixel(REAR, 4, 13, frc::Color(1, 0,0));

                setPixel(FRONT, 0, 5, frc::Color(1, 0,0));
                setPixel(FRONT, 0, 9, frc::Color(1, 0,0));
                setPixel(FRONT, 1, 6, frc::Color(1, 0,0));
                setPixel(FRONT, 1, 8, frc::Color(1, 0,0));
                setPixel(FRONT, 2, 7, frc::Color(1, 0,0));
                setPixel(FRONT, 3, 8, frc::Color(1, 0,0));
                setPixel(FRONT, 3, 6, frc::Color(1, 0,0));
                setPixel(FRONT, 4, 9, frc::Color(1, 0,0));
                setPixel(FRONT, 4, 5, frc::Color(1, 0,0));
                break;
            }
        }
    // }
    // else {
    //     for(int i = 0; i < 5; i++) {
    //         setRow(REAR, i, frc::Color(255,0,0));
    //     }

    //     for(int i = 0; i < 5; i++) {
    //         setRow(FRONT, i, frc::Color(255, 0, 255));
    //     }
    // }

    if (frc::DriverStation::GetInstance().IsDisabled()) {
        frc::Color color{1,0,0};
        frc::DriverStation::Alliance currentAlliance = frc::DriverStation::GetInstance().GetAlliance();

        if(currentAlliance == frc::DriverStation::kBlue) {
            color = frc::Color{0,0,1};
        }
        else if(currentAlliance == frc::DriverStation::kRed) {
            color = frc::Color{1,0,0};
        }

        if(!gyroCalibrating && currentAlliance != frc::DriverStation::kInvalid) {
            for(int i = 0; i < 5; i++) {
                setRow(REAR, i, color);
            }
        }
        if (!gyroCalibrating) {
            setRow(FRONT, 4, shooterExitSensorTripped ? frc::Color(1, 0, 0) : frc::Color(0, 1, 0));
            setRow(FRONT, 3, transitionSensorTripped ? frc::Color(1, 0, 0) : frc::Color(0, 1, 0));
            setRow(FRONT, 2, intakeBallCounterTripped ? frc::Color(1, 0, 0) : frc::Color(0, 1, 0));
            setRow(FRONT, 1, intakeZeroTripped ? frc::Color(1, 0, 0) : frc::Color(0, 1, 0));
            setRow(FRONT, 0, hangerZeroTripped ? frc::Color(1, 0, 0) : frc::Color(0, 1, 0));
        }
    }


    strip.SetData(stripBuffer);


    frc::DriverStation::Alliance alliance = frc::DriverStation::GetInstance().GetAlliance();
    const char* allianceName = "unknown";
    if(alliance == frc::DriverStation::kRed) allianceName = "kRed";
    else if(alliance == frc::DriverStation::kBlue) allianceName = "kBlue";
    else if(alliance == frc::DriverStation::kInvalid) allianceName = "kInvalid";
    frc::SmartDashboard::PutString("lights_alliance color", allianceName);    
    
}

void Lights::setPixel(LEDSection section, int row, int index, frc::Color color) {
    int targetIndex = 0;

    if(section == REAR) {
        if(index >= rearLengths[row]) {
            return;
        }
        
        if(row % 2 == 0) {// if even, add
            targetIndex = rearOffsets[row] + index;
        }
        else { // if odd
            targetIndex = rearOffsets[row] - index;
        }
    }
    else if(section == FRONT) {
        if(index >= frontLengths[row]) {
            return;
        }

        targetIndex = numLEDRear; // offset the rear LEDS

        if(row % 2 == 0) { // if even, subtract
            targetIndex += frontOffsets[row] - index;
        }
        else {
            targetIndex += frontOffsets[row] + index;
        }
    }

    stripBuffer[targetIndex].SetRGB(color.red * multiplier, color.green * multiplier, color.blue * multiplier);
}

void Lights::setRow(LEDSection section, int row, frc::Color color) {
    
    if(section == REAR) {
        row = 4-row;
        for(int i = 0; i < rearLengths[row]; i++) {
            setPixel(REAR, row, i, color);
        }
    }
    else if(section == FRONT) {
        for(int i = 0; i < frontLengths[row]; i++) {
            setPixel(FRONT, row, i, color);
        }
    }
}

void Lights::setBallCount(int count) {
    ballCount = count;
}

void Lights::setGyroCalibrating(bool cal)
{
    gyroCalibrating = cal;
    process();
}

void Lights::setControlPanelColor(bool cpActive, float r, float g, float b)
{
    this->cpActive = cpActive;
    this->cpR = r;
    this->cpG = g;
    this->cpB = b;
}

void Lights::setHangerZeroTripped(bool tripped)
{
    hangerZeroTripped = tripped;
}

void Lights::setIntakeZeroTripped(bool tripped)
{
    intakeZeroTripped = tripped;
}

void Lights::setIntakeBallCounterTripped(bool tripped)
{
    intakeBallCounterTripped = tripped;
}

void Lights::setTransitionSensorTripped(bool tripped)
{
    transitionSensorTripped = tripped;
}

void Lights::setShooterExitSensorTripped(bool tripped)
{
    shooterExitSensorTripped = tripped;
}
