/*----------------------------------------------------------------------------*/
/* Copyright (c) 2018 FIRST. All Rights Reserved.                             */
/* Open Source Software - may be modified and shared by FRC teams. The code   */
/* must be accompanied by the FIRST BSD license file in the root directory of */
/* the project.                                                               */
/*----------------------------------------------------------------------------*/

#include "Robot.h"

void Robot::RobotInit() {
    cs::UsbCamera camera = CameraServer::GetInstance()->StartAutomaticCapture();
    // camera.SetExposureManual(20);

    loopTimer.Reset();
    loopTimer.Start();
    autonomous.sendModesToDashboard();
}

void Robot::DisabledInit()
{
    gamePiece.setIntakeIdleMode(false);
    hang.setHangIdleMode(false);
    log.toggleWrite(false);
}

void Robot::DisabledPeriodic()
{
    controls.process();
    drive.process(); // handles velocity calculations

    processLights();

    drive.debug(&feedback);
    gamePiece.debug(&feedback);
    controlPanel.debug(&feedback);
    hang.debug(&feedback);
    controls.debug(&feedback);
}


void Robot::AutonomousInit()
{
    drive.reset();
    // XXX - get this from dashboard?
    gamePiece.reset(frc::SmartDashboard::GetNumber("thunderdashboard_starting_count", 3));
    controlPanel.reset();

    autonomous.getModeFromDashboard();
    autonomous.reset();

    gamePiece.setIntakeIdleMode(true);
    hang.setHangIdleMode(true);

    drive.calibrateGyro();
}
void Robot::AutonomousPeriodic() {
    autonomous.process();
    drive.process();
    gamePiece.process();
    
    gamePiece.debug(&feedback);
    processLights();

    drive.debug(&feedback);
    autonomous.debug(&feedback);
}

void Robot::TeleopInit() {
    drive.reset();
    gamePiece.reset(-1);
    controlPanel.reset();

    gamePiece.setIntakeIdleMode(true);
    hang.setHangIdleMode(true);
    drive.calibrateGyro();

    log.toggleWrite(false);
}
void Robot::TeleopPeriodic() {
    double startTime = loopTimer.Get();
    
    controls.process();
    drive.process();
    gamePiece.process();
    controlPanel.process();
    hang.process();

    hang.debug(&feedback);
    drive.debug(&feedback);
    gamePiece.debug(&feedback);
    controlPanel.debug(&feedback);
    controls.debug(&feedback);

    gamePiece.handleLog(&log);
    log.process();
    
    processLights();

    double endTime = loopTimer.Get();
    feedback.sendDouble("robot", "time start to end", endTime - startTime);
}

void Robot::TestInit() {
    
}
void Robot::TestPeriodic() {
   
}

void Robot::processLights() {
    gamePiece.lights(&lights);
    hang.lights(&lights);
    controlPanel.doLights(&lights);

    lights.process();
}

#ifndef RUNNING_FRC_TESTS
int main() { return frc::StartRobot<Robot>(); }
#endif
