#pragma once

#include "Feedback.h"
#include "Drive.h"
#include "GamePiece.h"
#include <iostream>
#include <frc/DriverStation.h>
#include <frc/Timer.h>
#include "frc/smartdashboard/Smartdashboard.h"
#include "networktables/NetworkTable.h"
#include "networktables/NetworkTableInstance.h"
#include "networktables/NetworkTableEntry.h"
#include "Limelight.h"

class Autonomous {
public:
    Autonomous(Drive* drive, GamePiece* gamePiece);

    Drive* drive;
    GamePiece* gamePiece;
    
    void process();

    void debug(Feedback* feedback);

    enum AutoMode{DONOTHING, CROSS_PORT, DRIVE_RIGHT_MORE, SHOOT_FROM_LINE, JUST_LINE, DRIVE_LEFT, FIVE_BALL_AUTO,
    CROSS_PORT_TRENCH,AUTONAVSLALOMPATH, PATH_A, BOUNCE_PATH, BARREL_RACING_PATH, PATH_B
    };
    AutoMode currentAutoMode = AutoMode::DONOTHING;

	void sendModesToDashboard();

	void getModeFromDashboard();

    void doStartupThings();

     void reset();


private:
    frc::Timer timer;
    int gameData;
    int step = 0;
    int trenchStep = -1;
    void doNothing();
    void shoot();
    void crossLine();
    void driveRightMore();
    void driveRight();
    void shootLine();
    void justLine();
    void driveLeft();
    void trenchBalls(); // Used after going to Power port
    void fiveBallAuto();
    void autoNavSlalomPath();
    void barrelRacingPath();
    
    //Galactic Search Path B
    void pathB();
    int pathBAction {};
    float pathBValue {};
    int intakeValue {};
    int pathBMode;

    // Galactic Search Path A
    void pathA();
    bool pathAEnd {}; // If we've completed the path
    int pathANumberOfCells {}; // Number of Power Cells we have in the robot
    int pathAPath {}; // Keeps track of which path to take
    int pathAAction {}; // The current action
    float pathAValue {}; // The move value for the current action
    bool pathADrove {};
    bool pathATimerOn {};

    // Autonav Bounce Path
    void bouncePath();
    bool bouncePathEnd {}; // If we've completed the path
    int bouncePathAction {}; // The current action
    float bouncePathValue {}; // The move value for the current action

    enum StartPosition {LEFT, RIGHT, CENTER};
    StartPosition position;

    void handleDashboardString(AutoMode mode, const char* description, char* buffer);

};


