//positive motorspeed may or may not be clockwise
#pragma once
#include "IOMap.h"
#include "ThunderSparkMax.h"
#include "rev/ColorSensorV3.h"
#include "Feedback.h"
#include <frc/DriverStation.h>
#include "Lights.h"

class ControlPanel {
    public:
        enum ControlPanelColor {RED, YELOW, GREEN, BLUE, DONT_KNOW};


        // Constructor
        ControlPanel();

        // Command:  Try to do position control
        // The target color is given by FMS and this
        // function reads the target
        // color from the driver station.
        // We keep trying to complete position control
        // until the target color is found or another
        // command is issued.
        void positionControl();
        // Command: Try to do rotation control
        // We keep trying to complete position control
        // until the target color is found or another
        // command is issued.
        void rotationControl();
        // Command: Stop any other running command
        void stop();
        // Command: run motor manually and fast (in case of broken sensor)
        void manualDriveFast();
        // Command: run motor manually and slow (in case of broken sensor)
        void manualDriveSlow();
        // Returns true if last position or rotation control command
        // has finished.  Returns false if last command was stop
        bool hasFinished();

        // Common subsystem function:
        // Process does the main work of the subsystem
        // Read in current status of the subsystem
        // and keep trying to complete the most recent
        // command given
        void process();

        // Reset the subsystem state back to as though the robot was just started up
        void reset();

        // Common subsystem function:
        // Debug feedback
        void debug(Feedback * feedback);

        void doLights(Lights *lights);

       
    private:
        enum ControlPanelPassed{NOTRED, NOTYELOW, NOTGREEN, NOTBLUE};
        ControlPanelPassed action;
        enum ControlPanelState {POSITION, ROTATION, STOPPED, MANUALFAST, MANUALSLOW};
        const char *colorToString(ControlPanelColor color);
        ControlPanelColor senseColor();
        bool rotationControlDone();
        bool overWheel();

        ThunderSparkMax *spinWheelMotor;
        #ifndef TEST_BOARD
        rev::ColorSensorV3 m_colorSensor{frc::I2C::Port::kOnboard};
        #endif
        ControlPanelState state = ControlPanelState::STOPPED;
        ControlPanelColor PositionControlColor;
        ControlPanelColor lastRawColorDetected;
        ControlPanelColor lastColorDetected;
        
        int sameColorCounted = 0;
        bool lastCommandFinished = false;
        bool sensorWasInRange = false;
        int lastProxValue = 0;
        double lastSensorRed = 0;
        double lastSensorGreen = 0;
        double lastSensorBlue = 0;
};
