#pragma once

#include <frc/AddressableLED.h>
#include <array>
#include <frc/Timer.h>

#include "IOMap.h"

const int numLEDRear = 79;
const int numLEDFront = 85;

const int numLEDTotal = numLEDRear + numLEDFront;

class Lights {
public:
    Lights();
    void process();

    void setBallCount(int count);
    void setControlPanelColor(bool cpActive, float r, float g, float b);
    void setGyroCalibrating(bool cal);
    void setHangerZeroTripped(bool tripped);
    void setIntakeZeroTripped(bool tripped);
    void setIntakeBallCounterTripped(bool tripped);
    void setTransitionSensorTripped(bool tripped);
    void setShooterExitSensorTripped(bool tripped);

private:
    frc::AddressableLED strip{PWM_LEDS};
    std::array<frc::AddressableLED::LEDData, numLEDTotal> stripBuffer;

    enum LEDSection {REAR, FRONT};

    void setPixel(LEDSection section, int row, int index, frc::Color color); // indexes are from the bottom and left
    void setRow(LEDSection section, int row, frc::Color color);
    

    bool hangerZeroTripped;
    bool intakeZeroTripped;
    bool intakeBallCounterTripped;
    bool transitionSensorTripped;
    bool shooterExitSensorTripped;

    int ballCount;    
    bool gyroCalibrating;
    bool cpActive;
    float cpR, cpG, cpB;
    frc::Timer timer;
};