#pragma once
#include "ThunderSparkMax.h"
#include "Feedback.h"
#include <ctre/Phoenix.h>
//#include <frc/Encoder.h>
#include "IOMap.h"
#include <frc/Servo.h>
#include <frc/Timer.h>
#include <frc/DigitalInput.h>
#include <frc/AnalogPotentiometer.h>
#include <frc/PIDController.h>
using namespace frc;


class Shooter
{
public:
  Shooter(bool rapidFire);

  /**
     * Reset shooter to initial state - turn off all motors
     */
  void reset();

  /**
     * Output debugging info
     */
  void debug(Feedback *feedback);

  /**
     * Process subsystem
     */
  void process();

  /**
     * Turn desired feeding of power cells on and off.
     * Shooter should turn on feeding of power cells when
     * shooter is at desired speed and continue to run it until
     * asked to stop (false).
     * Will never feed if setShooterSpinup is not enabled!
     */
  void feedPowerCell(bool feed);

  /**
     * Sets the desired speed of any forthcoming shots. Input values range from 0.0 to 1.0
     */
  void setShooterSpeed(double speed);
  
   /**
    * Causes shooter to use manually set speed instead of preset speeds
    */
  void enableManualControl(bool enabled);

  /**
     * Run shooter wheels at desired speed as consistently as possible.
     */
  void setShooterSpinup(bool shouldSpin);
  /**
    * Possible hood preset position
    */
  enum HoodPosition
  {
    GREEN, // Was PANEL
    YELLOW, // Was LINE
    RED, // Was PORT
    BLUE, // Was TARGET_CORNER
    MANUAL
  };
  /**
    * Sets hood to one of the preset positions
    */
  void hood(HoodPosition position);

  /**
   * Moves hood to manual position, used primarily for vision
   */
  void hoodManual(double hoodSpeed);


private:
  ThunderSparkMax *primerMotor;
  ThunderSparkMax *shooterMotorMaster;
  ThunderSparkMax *shooterMotorSlave;
  ThunderSparkMax *transitionMotor; 
  frc::Servo hoodServo{PWM_SHOOTER_HOOD};
  frc::AnalogPotentiometer hoodPotentiometer{HOOD_POTENTIOMETER};  

   ThunderSparkMaxCANPIDController* pidController;
   double desiredSpeed;
   bool manualControl = false;

   bool wantToShoot = false;
   bool wantToSpin = false; 

   HoodPosition hoodPos = GREEN;
   double hoodSpeedManual = .5;

   static const int kHistoryCount = 20;
   double velocityHistory[kHistoryCount];
   frc::Timer cooldown;

   void shiftVelocity(double velocity);

   /**
    * indicates if the shooter is spinning at a constant rate, aka is it at the driver-issued speed 
    */
   bool isAtVelocity();
   double hoodValue(double value);

   double reverseHoodEncoder(double value);
   


   double setPoint = 0;
   bool initDebug = true;
   bool rapidFiring = false;
};
